package com.tptj.demo.hg.controller.register;

import com.fanruan.api.decision.AuthorityKit;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.*;
import com.fr.decision.webservice.bean.entry.TemplateBean;
import com.fr.decision.webservice.bean.template.TemplateProductType;
import com.fr.license.function.VT4FR;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;


/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2021-03-22
 * 这个demo专门用来讲解FR特有的服务接口注解的用法和含义
 **/
@Controller
@RequestMapping(value = "/annotation/info")
@LoginStatusChecker(required = false)
public class AnnotationServer {

    @VisitRefer(refer= AuthorityKit.Management.USER)
    @LoginStatusChecker
    @RequestMapping(value = "/visit/refer",method = RequestMethod.GET)
    @ResponseBody
    public String visitRefer( @PathVariable("username") String username )throws Exception{
        return "@VisitRefer注解根据模块权限进行切面鉴权。没有用户管理权限的用户是看不到这个信息的";
    }

    @RequestMapping(value = "/fine/path/variable/{val}/{tp}",method = RequestMethod.GET)
    @ResponseBody
    public Response finePathVariable(@FinePathVariable("val") String val,
                                     @PathVariable("tp") String tp){
        //http://localhost:8075/webroot/decision/annotation/info/fine/path/variable/124%E4%B7%84%E4%B7%85ESD/124ASD
        Map<String,String> data = new HashMap<String,String>();
        data.put("val",val);
        data.put("tp",tp);
        data.put("doc","@FinePathVariable注解用于解析URI中符合FR特殊转义规则（EncodeDictionary中定义）的字符");
        data.put("/","\u4DC0\u4DC1");
        data.put(".","\u4DC2\u4DC3");
        data.put("\\\\","\u4DC4\u4DC5");
        data.put(" ","\u4DC6\u4DC7");
        return Response.ok(data);
    }

    @RequestMapping(value = "/lock",method = RequestMethod.GET)
    @Lock(Config.class)
    @ResponseBody
    public String lock(){
        return "@Lock注解用于并发修改配置时保证数据的一致性。";
    }


    @RequestMapping(value = "/template/auth",method = RequestMethod.GET)
    @ResponseBody
    @LoginStatusChecker
    @TemplateAuth(product= TemplateProductType.FINE_REPORT)
    public String templateAuth(){
        return "@TemplateAuth注解用于控制模板权类型是报表还是BI还是平台的一个标识，插件里面一般用不到";
    }

    @RequestMapping(value = "/function/support",method = RequestMethod.GET)
    @ResponseBody
    @FunctionSupport(function={VT4FR.RemoteDesign})
    public String functionSupport(){
        return "@FunctionSupport注解用于根据产品授权文件对请求进行授权访问，插件里面用不到";
    }

    @RequestMapping(value = "/decision/service/log",method = RequestMethod.GET)
    @DecisionServiceLog(resource="AnnotationServer[/annotation/info/decision/service/log]",detail="DecisionServiceLog注解demo")
    @ResponseBody
    public String decisionServiceLog(){
        return "@DecisionServiceLog本意是通过注解配合切面进行web服务的埋点。但是目前实际上这个切面功能并未实现";
    }

//    @RequestMapping(value = "/decision/service/log",method = RequestMethod.GET)
//    @DecisionCacheRefresh(cacheClass=DemoCache.class)
//    @ResponseBody
//    public String decisionCacheRefresh(){
//        return "访问方法加上@DecisionCacheRefresh注解之后，将被切面拦截，刷新对应的缓存项";
//    }

//    @RequestMapping(value = "/device/binding",method = RequestMethod.GET)
//    @ResponseBody
//    @LoginStatusChecker
//    @DeviceBinding(autoBindFirstDevice=true)
//    public String deviceBinding(){
//        return "@DeviceBinding注解用于移动端请求且产品开启了移动端设备绑定功能的前提下，如果请求接口申明了这个注解且auto_bind值为true" +
//                "则设备首次访问这个接口时会自动绑定移动端（绑定是指用户和设备绑定），插件里面目前用不到" ;
//    }


//    @RequestMapping(value = "/dcs/user/checker",method = RequestMethod.POST)
//    @DecisionUserChecker(targetIndex = {1})
//    @ResponseBody
//    @LoginStatusChecker
//    public String decisionUserChecker( @RequestBody UserBean userBean ){
//        return "@DecisionUserChecker注解用于帆软平台用户（RoleCheckerType.USER）/部门/角色管理权限鉴权的切面控制。" +
//                "通过对服务方法的入参下标申明，" +
//                "指定入参中需要确认权限的实体对象（该实体必须继承于RoleChecker，产品内部目前只有用户相关的bean）具体可以参照UserResource。" +
//                "插件中需要对帆软的用户进行管理权限判断时会用到，当然也可以定义";
//    }

//    @RequestMapping(value = "/decision/entry/checker",method = RequestMethod.GET)
//    @DecisionEntryChecker(targetIndex = 1)
//    @ResponseBody
//    @LoginStatusChecker
//    public String decisionEntryChecker( @PathVariable("directoryId") String directoryId ){
//        return "@DecisionEntryChecker注解用于平台鉴权功能中的每一种权限实体（用户编辑权限、用户授权权限、报表查看权限、" +
//                "报表编辑权限、报表授权权限等等 AuthorityStaticItemId.（VIEW/EDIT/AUTHORIZE）_AUTHORITY_TYPE）的用户鉴权" +
//                "通过对服务方法的入参下标申明，" +
//                "指定入参中需要确认权限的实体对象（该权限实体是登记在权限实体表中的一个ID）具体可以参照TemplateResource。" +
//                "插件中需要对帆软报表访问功能鉴权时会用到，实际需要使用的场景不多";
//    }

//    @RequestMapping(value = "/decision/controller/log",method = RequestMethod.GET)
//    @ResponseBody
//    @DecisionControllerLog
//    public String decisionControllerLog(){
//        return "@DecisionControllerLog注解作用于resource层方法上，用于平台管理日志埋点记录切面处理。插件中谨慎使用！";
//    }

//    @RequestMapping(value = "/dcs/cfg/checker/{version}",method = RequestMethod.POST)
//    @ResponseBody
//    @DecisionConfigChecker
//    @LoginStatusChecker
//    public String decisionConfigChecker(HttpServletRequest req, HttpServletResponse res,
//                                        @PathVariable("version") String version){
//        return "@DecisionConfigChecker,配置项检查注解，作用于resource层方法上。加上该注解的方法，只有管理员有权限访问";
//    }

}
